"""Conftest."""

import pytest

from typing import Any, Sequence, TypeVar, Tuple

from spec.default import get


@pytest.fixture
def default_spec() -> Sequence[Tuple[str, TypeVar, Any]]:
    """Spec default values."""
    return (
        ('ENVIRONMENT', str, get('ENVIRONMENT')),
        ('SERVICE_HOST', str, get('SERVICE_HOST')),
        ('SERVICE_PORT', int, get('SERVICE_PORT')),
        ('SERVICE_NAME', str, get('SERVICE_NAME')),
        ('SERVICE_SCHEME_SECURE', str, get('SERVICE_SCHEME_SECURE')),
        ('SERVICE_SCHEME_INSECURE', str, get('SERVICE_SCHEME_INSECURE')),
        ('LOG_LEVEL', str, get('LOG_LEVEL')),
        ('LOG_CONFIG_PATH', str, get('LOG_CONFIG_PATH')),
        ('ENV_FILE', str, get('ENV_FILE')),
        ('SENTRY_DSN', str, get('SENTRY_DSN')),
        ('POLICY_SERVICE_WORKERS', str, get('POLICY_SERVICE_WORKERS')),
        ('POLICY_REQUEST_TIMEOUT', str, get('POLICY_REQUEST_TIMEOUT')),
        ('POLICY_REQUEST_RETRY', str, get('POLICY_REQUEST_RETRY')),
        ('POLICY_SCHEDULER_ENABLED', str, get('POLICY_SCHEDULER_ENABLED')),
        ('POLICY_SCHEDULER_PERSISTENT', str, get('POLICY_SCHEDULER_PERSISTENT')),
        ('POLICY_SCHEDULER_WORKERS', str, get('POLICY_SCHEDULER_WORKERS')),
        ('POLICY_SCHEDULER_INSTANCES', str, get('POLICY_SCHEDULER_INSTANCES')),
        ('POLICY_SCHEDULER_HOST', str, get('POLICY_SCHEDULER_HOST')),
        ('POLICY_SCHEDULER_PORT', str, get('POLICY_SCHEDULER_PORT')),
        ('POLICY_SCHEDULER_DB', str, get('POLICY_SCHEDULER_DB')),
        ('POLICY_SCHEDULER_COALESCE', str, get('POLICY_SCHEDULER_COALESCE')),
        ('I18N_LANG', str, get('I18N_LANG')),
        ('I18N_SUPPORT', str, get('I18N_SUPPORT')),
        ('I18N_LOCALES', str, get('I18N_LOCALES')),
    )


@pytest.fixture
def pyproject(mocker):
    """Mock pyproject."""
    fake_pyproject = {
        'tool': {
            'poetry': {
                'name': 'service-name',
                'version': '1.2.3',
                'description': 'Service description',
            },
        },
    }

    mocker.patch('spec.fn.load_pyproject', return_value=fake_pyproject)
    return fake_pyproject


@pytest.fixture
def k8s_running(mocker):
    """Mock k8s running."""
    mocker.patch('spec.fn.on_k8s', return_value=True)


@pytest.fixture
def env_sentry_dsn(monkeypatch) -> str:
    """Mock k8s running."""
    dsn = 'https://valid.sentry.dsn'
    monkeypatch.setenv('SENTRY_DSN', dsn)
    yield dsn
    monkeypatch.delenv('SENTRY_DSN', raising=False)
